#! /usr/bin/env python3

"""
Software License Agreement (BSD License)

 Point Cloud Library (PCL) - www.pointclouds.org
 Copyright (c) 2018-, Open Perception, Inc.

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.
  * Neither the name of the copyright holder(s) nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

"""

import json
import argparse
from pathlib import Path

import requests
import re


CATEGORIES = {
    "new feature": ("New features", "added to PCL"),
    "deprecation": (
        "Deprecation",
        "of public APIs, scheduled to be removed after two minor releases",
    ),
    "removal": ("Removal", "of the public APIs deprecated in previous releases"),
    "behavior change": ("Behavior changes", "in classes, apps, or tools",),
    "API break": (
        "API changes",
        "that did not go through the proper deprecation and removal cycle",
    ),
    "ABI break": ("ABI changes", "that are still API compatible",),
    "fix": (None, None),
    "enhancement": (None, None),
}


MODULES = {
    "cmake": "CMake",
    "2d": "libpcl_2d",
    "common": "libpcl_common",
    "cuda": "libpcl_cuda",
    "features": "libpcl_features",
    "filters": "libpcl_filters",
    "geometry": "libpcl_geometry",
    "gpu": "libpcl_gpu",
    "io": "libpcl_io",
    "kdtree": "libpcl_kdtree",
    "keypoints": "libpcl_keypoints",
    "ml": "libpcl_ml",
    "octree": "libpcl_octree",
    "outofcore": "libpcl_outofcore",
    "people": "libpcl_people",
    "recognition": "libpcl_recognition",
    "registration": "libpcl_registration",
    "sample_consensus": "libpcl_sample_consensus",
    "search": "libpcl_search",
    "segmentation": "libpcl_segmentation",
    "simulation": "libpcl_simulation",
    "stereo": "libpcl_stereo",
    "surface": "libpcl_surface",
    "visualization": "libpcl_visualization",
    "apps": "PCL Apps",
    "docs": "PCL Docs",
    "tutorials": "PCL Tutorials",
    "examples": "PCL Examples",
    "tests": "PCL Tests",
    "tools": "PCL Tools",
    "ci": "CI",
    None: "Uncategorized",
}


def fetch_latest_release_date():
    """
    Fetch the date when the latest release was created.
    """
    url = "https://api.github.com/repos/PointCloudLibrary/pcl/releases/latest"
    response = requests.get(url)
    response.raise_for_status()
    return response.json()["created_at"]


def fetch_pr_since(start_date):
    """
    Fetch data of PRs merged after a given start date.
    """
    url = f"https://api.github.com/search/issues?q=is:pr+repo:PointCloudLibrary/pcl+merged:>={start_date}"
    pr_data = list()
    page = 1
    while True:
        response = requests.get(f"{url}&page={page}&per_page=100")
        response.raise_for_status()
        data = response.json()
        total_count = data["total_count"]
        pr_data.extend(data["items"])
        page += 1
        if len(pr_data) == total_count:
            break
    return pr_data


def filter_labels(labels, prefix):
    """
    Filter a given list of PR labels, keeping only those starting with a given prefix.
    The prefix is stripped from the labels.
    """
    return [
        label["name"][len(prefix) :]
        for label in labels
        if label["name"].startswith(prefix)
    ]


def strip_leading_tag(text):
    """
    >>> strip_leading_tag("[text] larger text")
    'larger text'
    >>> strip_leading_tag("no tag text")
    'no tag text'
    """
    if len(text) == 0 or text[0] != '[':
        return text
    pattern = re.compile('\[.*\]\s*')
    match = pattern.match(text)
    return text[match.end():] if match else text


def make_pr_bullet_point(pr, prefix=None):
    ref = "[#{0}](https://github.com/PointCloudLibrary/pcl/pull/{0})".format(
        pr["number"]
    )

    tags = ""
    if prefix in ("modules", "categories"):
        tags = "".join(["[" + k + "]" for k in pr[prefix]])
    if tags:
        tags = "**" + tags + "** "

    return f"* {tags}{pr['title']} [{ref}]"


def generate_category_section(key, prs):
    section = list()
    filtered_prs = [pr for pr in prs if key in pr["categories"]]
    title, description = CATEGORIES[key]
    if filtered_prs and title:
        section += [f"\n**{title}** *{description}*\n"]
        section += [make_pr_bullet_point(pr, "modules") for pr in filtered_prs]
    return section


def generate_module_section(key, prs):
    section = list()
    filtered_prs = [pr for pr in prs if key in pr["modules"]]
    title = MODULES[key]
    if filtered_prs and title:
        section += [f"\n#### {title}:\n"]
        section += [make_pr_bullet_point(pr, "categories") for pr in filtered_prs]
    return section


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="""
    Generate changelog from pull requests merged after the latest release.
    """,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    cache_grp = parser.add_mutually_exclusive_group()
    cache_grp.add_argument("--save", type=Path, help="Save PR data fetched from GitHub")
    cache_grp.add_argument("--load", type=Path, help="Load PR data from a file")
    parser.add_argument(
        "--with-misc",
        "-m",
        action="store_true",
        help=(
            "Add section with miscellaneous PRs that are not important enough to be "
            "included in the official changelog"
        ),
    )
    args = parser.parse_args()

    if args.load:
        with open(args.load) as fp:
            pr_data = json.loads(fp.read())
    else:
        date = fetch_latest_release_date()
        pr_data = fetch_pr_since(date)
        if args.save:
            with open(args.save, "w") as fp:
                fp.write(json.dumps(pr_data))

    selected_prs = list()
    excluded_prs = list()
    for pr in sorted(pr_data, key=lambda d: d["closed_at"]):
        categories = filter_labels(pr["labels"], "changelog: ")
        title = strip_leading_tag(pr["title"])
        pr_info = {
            "number": pr["number"],
            "title": title,
            "modules": filter_labels(pr["labels"], "module: "),
            "categories": [g for g in categories if g not in ["fix", "enhancement"]],
        }
        if categories:
            selected_prs.append(pr_info)
        else:  # exclude PRs not tagged with any changelog label
            excluded_prs.append(pr_info)

    clog = list()

    clog += ["\n### Notable changes"]
    for k in CATEGORIES.keys():
        clog.extend(generate_category_section(k, selected_prs))

    clog += ["\n### Changes grouped by module"]
    for k in MODULES.keys():
        clog.extend(generate_module_section(k, selected_prs))

    if args.with_misc:
        clog += ["\n### Miscellaneous PRs excluded from changelog\n"]
        for pr in excluded_prs:
            clog += [make_pr_bullet_point(pr)]

    print("\n".join(clog))



#!/usr/bin/env python
# file: merge_cmake_install.py

import sys, math
import fnmatch
import os
import shutil

if len(sys.argv) != 2: 
  # stop the program and print an error message
  sys.exit("Must provide a cmake binary folder")

base_folder = sys.argv[1]

string_to_remove_debug = "IF(\"${CMAKE_INSTALL_CONFIG_NAME}\" MATCHES \"^([Dd][Ee][Bb][Uu][Gg])$\")"
string_to_remove_release = "IF(\"${CMAKE_INSTALL_CONFIG_NAME}\" MATCHES \"^([Rr][Ee][Ll][Ee][Aa][Ss][Ee])$\")"

matches = []
for root, dirnames, filenames in os.walk(base_folder):
  for filename in fnmatch.filter(filenames, 'cmake_install.cmake'):
      matches.append(os.path.join(root, filename))
	  
for one_match in matches:
	#print one_match, "\n"
	shutil.move( one_match, one_match+"~" )
	destination= open( one_match, "w" )
	source= open( one_match+"~", "r" )
	for line in source:
		if string_to_remove_debug in line:
			destination.write( "\n" )
		elif string_to_remove_release in line:
			destination.write( "\n" )
		else:
			destination.write( line )
	source.close()
	destination.close()


##
#  Software License Agreement (BSD License)
#
#  Point Cloud Library (PCL) - www.pointclouds.org
#  Copyright (c) 2009-2012, Willow Garage, Inc.
#  Copyright (c) 2012-, Open Perception, Inc.
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#  #  Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  #  Redistributions in binary form must reproduce the above
#     copyright notice, this list of conditions and the following
#     disclaimer in the documentation and/or other materials provided
#     with the distribution.
#  #  Neither the name of the copyright holder(s) nor the names of its
#     contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.
#

import os

filename  = os.path.join (os.path.dirname (__file__), 'mesh_circulators.h')

class Class:
    def __init__ (self, value_prefix, value_type, around_type, current_he, deref, inc1, inc2, dec1, dec2):
        self.value_prefix  = value_prefix
        self.value_type  = value_type
        self.around_type = around_type
        self.current_he  = current_he
        self.deref       = deref
        self.inc1        = inc1
        self.inc2        = inc2
        self.dec1        = dec1
        self.dec2        = dec2
        self.docstring   = 'TODO: Add documentation!'

classes = []
classes.append (Class (''        , 'Vertex'  , 'Vertex', 'Outgoing', 'TerminatingVertex', 'Opposite', 'Next'    , 'Prev'    , 'Opposite')) # 0
classes.append (Class ('Outgoing', 'HalfEdge', 'Vertex', 'Outgoing', ''                 , 'Opposite', 'Next'    , 'Prev'    , 'Opposite')) # 1
classes.append (Class ('Incoming', 'HalfEdge', 'Vertex', 'Incoming', ''                 , 'Next'    , 'Opposite', 'Opposite', 'Prev'    )) # 2
classes.append (Class (''        , 'Face'    , 'Vertex', 'Outgoing', 'Face'             , 'Opposite', 'Next'    , 'Prev'    , 'Opposite')) # 3
classes.append (Class (''        , 'Vertex'  , 'Face'  , 'Inner'   , 'TerminatingVertex', 'Next'    , ''        , 'Prev'    , ''        )) # 4
classes.append (Class ('Inner'   , 'HalfEdge', 'Face'  , 'Inner'   , ''                 , 'Next'    , ''        , 'Prev'    , ''        )) # 5
classes.append (Class ('Outer'   , 'HalfEdge', 'Face'  , 'Inner'   , 'OppositeHalfEdge' , 'Next'    , ''        , 'Prev'    , ''        )) # 6
classes.append (Class (''        , 'Face'    , 'Face'  , 'Inner'   , 'OppositeFace'     , 'Next'    , ''        , 'Prev'    , ''        )) # 7

classes [0].docstring = 'Circulates counter-clockwise around a vertex and returns an index to the terminating vertex of the outgoing half-edge (the target).'
classes [1].docstring = 'Circulates counter-clockwise around a vertex and returns an index to the outgoing half-edge (the target).'
classes [2].docstring = 'Circulates counter-clockwise around a vertex and returns an index to the incoming half-edge (the target).'
classes [3].docstring = 'Circulates counter-clockwise around a vertex and returns an index to the face of the outgoing half-edge (the target).'
classes [4].docstring = 'Circulates clockwise around a face and returns an index to the terminating vertex of the inner half-edge (the target).'
classes [5].docstring = 'Circulates clockwise around a face and returns an index to the inner half-edge (the target).'
classes [6].docstring = 'Circulates clockwise around a face and returns an index to the outer half-edge (the target).'
classes [7].docstring = 'Circulates clockwise around a face and returns an index to the face of the outer half-edge (the target).'

################################################################################

f = open (filename, 'w')

f.write ('/*\n')
f.write (' * Software License Agreement (BSD License)\n')
f.write (' *\n')
f.write (' * Point Cloud Library (PCL) - www.pointclouds.org\n')
f.write (' * Copyright (c) 2009-2012, Willow Garage, Inc.\n')
f.write (' * Copyright (c) 2012-, Open Perception, Inc.\n')
f.write (' *\n')
f.write (' * All rights reserved.\n')
f.write (' *\n')
f.write (' * Redistribution and use in source and binary forms, with or without\n')
f.write (' * modification, are permitted provided that the following conditions\n')
f.write (' * are met:\n')
f.write (' *\n')
f.write (' *  * Redistributions of source code must retain the above copyright\n')
f.write (' *    notice, this list of conditions and the following disclaimer.\n')
f.write (' *  * Redistributions in binary form must reproduce the above\n')
f.write (' *    copyright notice, this list of conditions and the following\n')
f.write (' *    disclaimer in the documentation and/or other materials provided\n')
f.write (' *    with the distribution.\n')
f.write (' *  * Neither the name of the copyright holder(s) nor the names of its\n')
f.write (' *    contributors may be used to endorse or promote products derived\n')
f.write (' *    from this software without specific prior written permission.\n')
f.write (' *\n')
f.write (' * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS\n')
f.write (' * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT\n')
f.write (' * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS\n')
f.write (' * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE\n')
f.write (' * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,\n')
f.write (' * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,\n')
f.write (' * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;\n')
f.write (' * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\n')
f.write (' * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT\n')
f.write (' * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN\n')
f.write (' * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE\n')
f.write (' * POSSIBILITY OF SUCH DAMAGE.\n')
f.write (' *\n')
f.write (' * $Id$\n')
f.write (' *\n')
f.write (' */\n\n')

f.write ("// NOTE: This file has been created with 'pcl_src/geometry/include/pcl/geometry/mesh_circulators.py'\n\n")

f.write ('#ifndef PCL_GEOMETRY_MESH_CIRCULATORS_H\n')
f.write ('#define PCL_GEOMETRY_MESH_CIRCULATORS_H\n\n')

f.write ('#include <pcl/geometry/boost.h>\n')
f.write ('#include <pcl/geometry/mesh_indices.h>\n\n')

for c in classes:

    value_prefix    = c.value_prefix
    value_name      = c.value_prefix + c.value_type
    value_type      = c.value_type
    around_type     = c.around_type
    around_obj      = around_type.lower ()
    around_idx      = 'idx_' + around_obj
    current_he      = c.current_he
    current_he_idx  = 'idx_' + current_he.lower () + '_half_edge'
    current_he_idx_ = current_he_idx + '_'
    deref           = c.deref
    inc1            = c.inc1
    inc2            = c.inc2
    dec1            = c.dec1
    dec2            = c.dec2

    class_name      = value_name + 'Around' + around_type + 'Circulator'

    placeholder_at  = ' ' * (len (around_type) - 3)
    placeholder_cn  = ' ' *  len (class_name)
    placeholder_gt  = ' ' * (len (current_he_idx) - 5)

    f.write ('////////////////////////////////////////////////////////////////////////////////\n')
    f.write ('// ' + class_name + '\n')
    f.write ('////////////////////////////////////////////////////////////////////////////////\n\n')

    f.write ('namespace pcl\n')
    f.write ('{\n')
    f.write ('  namespace geometry\n')
    f.write ('  {\n')

    f.write ('    /** \\brief ' + c.docstring + ' The best way to declare the circulator is to use the method pcl::geometry::MeshBase::get' + class_name + ' ().\n')
    f.write ("      * \\tparam MeshT Mesh to which this circulator belongs to.\n")
    f.write ("      * \\note The circulator can't be used to change the connectivity in the mesh (only const circulators are valid).\n")
    f.write ('      * \\author Martin Saelzle\n')
    f.write ('      * \ingroup geometry\n')
    f.write ('      */\n')

    f.write ('    template <class MeshT>\n')
    f.write ('    class ' + class_name + '\n')
    f.write ('        : boost::equality_comparable <pcl::geometry::' + class_name + ' <MeshT>\n')
    f.write ('        , boost::unit_steppable      <pcl::geometry::' + class_name + ' <MeshT>\n')
    f.write ('        > >\n')
    f.write ('    {\n')
    f.write ('      public:\n\n')

    f.write ('        typedef boost::equality_comparable <pcl::geometry::' + class_name + ' <MeshT>\n')
    f.write ('              , boost::unit_steppable      <pcl::geometry::' + class_name + ' <MeshT> > > Base;\n')
    f.write ('        typedef pcl::geometry::' + class_name + ' <MeshT> Self;\n\n')

    f.write ('        typedef MeshT Mesh;\n')
    if value_type != 'HalfEdge':
        f.write ('        typedef typename Mesh::' + value_type + 'Index ' + value_type + 'Index;\n')
    if around_type != value_type:
        f.write ('        typedef typename Mesh::' + around_type + 'Index ' + around_type + 'Index;\n')
    f.write ('        typedef typename Mesh::HalfEdgeIndex HalfEdgeIndex;\n\n')

    f.write ('        /** \\brief Constructor resulting in an invalid circulator. */\n')
    f.write ('        ' + class_name + ' ()\n')
    f.write ('          : mesh_ ' + placeholder_gt + ' (NULL),\n')
    f.write ('            '       + current_he_idx_ + ' ()\n')
    f.write ('        {\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Construct from the ' + around_obj + ' around which we want to circulate. */\n')
    f.write ('        ' + class_name     + ' (const '      + around_type    + 'Index& ' + around_idx + ',\n')
    f.write ('        ' + placeholder_cn + '  Mesh*const ' + placeholder_at + '     '   + 'mesh)\n')
    f.write ('          : mesh_ ' + placeholder_gt + ' (mesh),\n')
    f.write ('            '       + current_he_idx_ + ' (mesh->get' + current_he + 'HalfEdgeIndex (' + around_idx + '))\n')
    f.write ('        {\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Construct directly from the ' + current_he.lower () + ' half-edge. */\n')
    f.write ('        ' + class_name     + ' (const HalfEdgeIndex& ' + current_he_idx  + ',\n')
    f.write ('        ' + placeholder_cn + '  Mesh*const           ' + 'mesh)\n')
    f.write ('          : mesh_ ' + placeholder_gt + ' (mesh),\n')
    f.write ('            '       + current_he_idx_ + ' (' + current_he_idx + ')\n')
    f.write ('        {\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Check if the circulator is valid.\n')
    f.write ('          * \\warning Does NOT check if the stored mesh pointer is valid. You have to ensure this yourself when constructing the circulator. */\n')
    f.write ('        inline bool\n')
    f.write ('        isValid () const\n')
    f.write ('        {\n')
    f.write ('          return (' + current_he_idx_ + '.isValid ());\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Comparison operators (with boost::operators): == !=\n')
    f.write ('          * \\warning Does NOT check if the circulators belong to the same mesh. Please check this yourself. */\n')
    f.write ('        inline bool\n')
    f.write ('        operator == (const Self& other) const\n')
    f.write ('        {\n')
    f.write ('          return (' + current_he_idx_ + ' == other.' + current_he_idx_ + ');\n')
    f.write ('        }\n\n')

    tmp = 'mesh_->get' + inc1 + 'HalfEdgeIndex (' + current_he_idx_ + ')'
    if inc2:
        tmp = 'mesh_->get' + inc2 + 'HalfEdgeIndex (' + tmp + ')'

    f.write ('        /** \\brief Increment operators (with boost::operators): ++ (pre and post) */\n')
    f.write ('        inline Self&\n')
    f.write ('        operator ++ ()\n')
    f.write ('        {\n')
    f.write ('          ' + current_he_idx_ + ' = ' + tmp + ';\n')
    f.write ('          return (*this);\n')
    f.write ('        }\n\n')

    tmp = 'mesh_->get' + dec1 + 'HalfEdgeIndex (' + current_he_idx_ + ')'
    if dec2:
        tmp = 'mesh_->get' + dec2 + 'HalfEdgeIndex (' + tmp + ')'

    f.write ('        /** \\brief Decrement operators (with boost::operators): -- (pre and post) */\n')
    f.write ('        inline Self&\n')
    f.write ('        operator -- ()\n')
    f.write ('        {\n')
    f.write ('          ' + current_he_idx_ + ' = ' + tmp + ';\n')
    f.write ('          return (*this);\n')
    f.write ('        }\n\n')

    if deref:
        tmp = 'mesh_->get' + deref + 'Index (' + current_he_idx_ + ')'
    else:
        tmp = current_he_idx_
    tgt = 'half-edge' if value_type=='HalfEdge' else value_type.lower ()
    tgt = value_prefix.lower () + (' ' if value_prefix else 'target ')  + tgt

    f.write ('        /** \\brief Get the index to the ' + tgt + '. */\n')
    f.write ('        inline ' + value_type + 'Index\n')
    f.write ('        getTargetIndex () const\n')
    f.write ('        {\n')
    f.write ('          return (' + tmp + ');\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Get the half-edge that is currently stored in the circulator. */\n')
    f.write ('        inline HalfEdgeIndex\n')
    f.write ('        getCurrentHalfEdgeIndex () const\n')
    f.write ('        {\n')
    f.write ('          return (' + current_he_idx_ + ');\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief The mesh to which this circulator belongs to. */\n')
    f.write ('        Mesh* mesh_;\n\n')

    f.write ('        /** \\brief The ' + current_he.lower () + ' half-edge of the ' + around_obj + ' around which we want to circulate. */\n')
    f.write ('        HalfEdgeIndex ' + current_he_idx_ + ';\n')
    f.write ('    };\n')

    f.write ('  } // End namespace geometry\n')
    f.write ('} // End namespace pcl\n\n')

f.write ('#endif // PCL_GEOMETRY_MESH_CIRCULATORS_H\n')

f.close()



##
#  Software License Agreement (BSD License)
#
#  Point Cloud Library (PCL) - www.pointclouds.org
#  Copyright (c) 2009-2012, Willow Garage, Inc.
#  Copyright (c) 2012-, Open Perception, Inc.
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#  #  Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  #  Redistributions in binary form must reproduce the above
#     copyright notice, this list of conditions and the following
#     disclaimer in the documentation and/or other materials provided
#     with the distribution.
#  #  Neither the name of the copyright holder(s) nor the names of its
#     contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.
#

import os

filename    = os.path.join (os.path.dirname (__file__), 'mesh_indices.h')
class_names = ['VertexIndex', 'HalfEdgeIndex', 'EdgeIndex', 'FaceIndex']

################################################################################

f = open (filename, 'w')

f.write ('/*\n')
f.write (' * Software License Agreement (BSD License)\n')
f.write (' *\n')
f.write (' * Point Cloud Library (PCL) - www.pointclouds.org\n')
f.write (' * Copyright (c) 2009-2012, Willow Garage, Inc.\n')
f.write (' * Copyright (c) 2012-, Open Perception, Inc.\n')
f.write (' *\n')
f.write (' * All rights reserved.\n')
f.write (' *\n')
f.write (' * Redistribution and use in source and binary forms, with or without\n')
f.write (' * modification, are permitted provided that the following conditions\n')
f.write (' * are met:\n')
f.write (' *\n')
f.write (' *  * Redistributions of source code must retain the above copyright\n')
f.write (' *    notice, this list of conditions and the following disclaimer.\n')
f.write (' *  * Redistributions in binary form must reproduce the above\n')
f.write (' *    copyright notice, this list of conditions and the following\n')
f.write (' *    disclaimer in the documentation and/or other materials provided\n')
f.write (' *    with the distribution.\n')
f.write (' *  * Neither the name of the copyright holder(s) nor the names of its\n')
f.write (' *    contributors may be used to endorse or promote products derived\n')
f.write (' *    from this software without specific prior written permission.\n')
f.write (' *\n')
f.write (' * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS\n')
f.write (' * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT\n')
f.write (' * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS\n')
f.write (' * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE\n')
f.write (' * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,\n')
f.write (' * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,\n')
f.write (' * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;\n')
f.write (' * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\n')
f.write (' * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT\n')
f.write (' * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN\n')
f.write (' * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE\n')
f.write (' * POSSIBILITY OF SUCH DAMAGE.\n')
f.write (' *\n')
f.write (' * $Id$\n')
f.write (' *\n')
f.write (' */\n\n')

f.write ("// NOTE: This file has been created with 'pcl_src/geometry/include/pcl/geometry/mesh_indices.py'\n\n")

f.write ('#ifndef PCL_GEOMETRY_MESH_INDICES_H\n')
f.write ('#define PCL_GEOMETRY_MESH_INDICES_H\n\n')

f.write ('#include <iostream>\n\n')

f.write ('#include <pcl/geometry/boost.h>\n\n')

for cn in class_names:

    f.write ('////////////////////////////////////////////////////////////////////////////////\n')
    f.write ('// ' + cn + '\n')
    f.write ('////////////////////////////////////////////////////////////////////////////////\n\n')

    f.write ('namespace pcl\n')
    f.write ('{\n')
    f.write ('  namespace geometry\n')
    f.write ('  {\n')
    f.write ('    /** \\brief Index used to access elements in the half-edge mesh. It is basically just a wrapper around an integer with a few added methods.\n')
    f.write ('      * \\author Martin Saelzle\n')
    f.write ('      * \ingroup geometry\n')
    f.write ('      */\n')
    f.write ('    class ' + cn + '\n')
    f.write ('        : boost::totally_ordered <pcl::geometry::' + cn + ' // < > <= >= == !=\n')
    f.write ('        , boost::unit_steppable  <pcl::geometry::' + cn + ' // ++ -- (pre and post)\n')
    f.write ('        , boost::additive        <pcl::geometry::' + cn + ' // += + -= -\n')
    f.write ('        > > >\n')
    f.write ('    {\n')
    f.write ('      public:\n\n')

    f.write ('        typedef boost::totally_ordered <pcl::geometry::' + cn + ',\n')
    f.write ('                boost::unit_steppable  <pcl::geometry::' + cn + ',\n')
    f.write ('                boost::additive        <pcl::geometry::' + cn + '> > > Base;\n')
    f.write ('        typedef pcl::geometry::' + cn + '                              Self;\n\n')

    f.write ('        /** \\brief Constructor. Initializes with an invalid index. */\n')
    f.write ('        ' + cn + ' ()\n')
    f.write ('          : index_ (-1)\n')
    f.write ('        {\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Constructor.\n')
    f.write ('          * \param[in] index The integer index.\n')
    f.write ('          */\n')
    f.write ('        explicit ' + cn + ' (const int index)\n')
    f.write ('          : index_ (index)\n')
    f.write ('        {\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Returns true if the index is valid. */\n')
    f.write ('        inline bool\n')
    f.write ('        isValid () const\n')
    f.write ('        {\n')
    f.write ('          return (index_ >= 0);\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Invalidate the index. */\n')
    f.write ('        inline void\n')
    f.write ('        invalidate ()\n')
    f.write ('        {\n')
    f.write ('          index_ = -1;\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Get the index. */\n')
    f.write ('        inline int\n')
    f.write ('        get () const\n')
    f.write ('        {\n')
    f.write ('          return (index_);\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Set the index. */\n')
    f.write ('        inline void\n')
    f.write ('        set (const int index)\n')
    f.write ('        {\n')
    f.write ('          index_ = index;\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Comparison operators (with boost::operators): < > <= >= */\n')
    f.write ('        inline bool\n')
    f.write ('        operator < (const Self& other) const\n')
    f.write ('        {\n')
    f.write ('          return (this->get () < other.get ());\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Comparison operators (with boost::operators): == != */\n')
    f.write ('        inline bool\n')
    f.write ('        operator == (const Self& other) const\n')
    f.write ('        {\n')
    f.write ('          return (this->get () == other.get ());\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Increment operators (with boost::operators): ++ (pre and post) */\n')
    f.write ('        inline Self&\n')
    f.write ('        operator ++ ()\n')
    f.write ('        {\n')
    f.write ('          ++index_;\n')
    f.write ('          return (*this);\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Decrement operators (with boost::operators): \-\- (pre and post) */\n')
    f.write ('        inline Self&\n')
    f.write ('        operator -- ()\n')
    f.write ('        {\n')
    f.write ('          --index_;\n')
    f.write ('          return (*this);\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Addition operators (with boost::operators): + += */\n')
    f.write ('        inline Self&\n')
    f.write ('        operator += (const Self& other)\n')
    f.write ('        {\n')
    f.write ('          index_ += other.get ();\n')
    f.write ('          return (*this);\n')
    f.write ('        }\n\n')

    f.write ('        /** \\brief Subtraction operators (with boost::operators): - -= */\n')
    f.write ('        inline Self&\n')
    f.write ('        operator -= (const Self& other)\n')
    f.write ('        {\n')
    f.write ('          index_ -= other.get ();\n')
    f.write ('          return (*this);\n')
    f.write ('        }\n\n')

    f.write ('      private:\n\n')

    f.write ('        /** \\brief Stored index. */\n')
    f.write ('        int index_;\n\n')

    f.write ('        friend std::istream&\n')
    f.write ('        operator >> (std::istream& is, pcl::geometry::' + cn + '& index);\n')
    f.write ('    };\n\n')

    f.write ('    /** \\brief ostream operator. */\n')
    f.write ('    inline std::ostream&\n')
    f.write ('    operator << (std::ostream& os, const pcl::geometry::' + cn + '& index)\n')
    f.write ('    {\n')
    f.write ('      return (os << index.get ());\n')
    f.write ('    }\n\n')

    f.write ('    /** \\brief istream operator. */\n')
    f.write ('    inline std::istream&\n')
    f.write ('    operator >> (std::istream& is, pcl::geometry::' + cn + '& index)\n')
    f.write ('    {\n')
    f.write ('      return (is >> index.index_);\n')
    f.write ('    }\n\n')

    f.write ('  } // End namespace geometry\n')
    f.write ('} // End namespace pcl\n\n')

f.write ('////////////////////////////////////////////////////////////////////////////////\n')
f.write ('// Conversions\n')
f.write ('////////////////////////////////////////////////////////////////////////////////\n\n')

f.write ('namespace pcl\n')
f.write ('{\n')
f.write ('  namespace geometry\n')
f.write ('  {\n')
f.write ('    /** \\brief Convert the given half-edge index to an edge index. */\n')
f.write ('    inline pcl::geometry::EdgeIndex\n')
f.write ('    toEdgeIndex (const HalfEdgeIndex& index)\n')
f.write ('    {\n')
f.write ('      return (index.isValid () ? EdgeIndex (index.get () / 2) : EdgeIndex ());\n')
f.write ('    }\n\n')

f.write ('    /** \\brief Convert the given edge index to a half-edge index.\n')
f.write ('      * \\param[in] get_first The first half-edge of the edge is returned if this variable is true; elsewise the second.\n')
f.write ('      */\n')
f.write ('    inline pcl::geometry::HalfEdgeIndex\n')
f.write ('    toHalfEdgeIndex (const EdgeIndex& index, const bool get_first=true)\n')
f.write ('    {\n')
f.write ('      return (index.isValid () ? HalfEdgeIndex (index.get () * 2 + static_cast <int> (!get_first)) : HalfEdgeIndex ());\n')
f.write ('    }\n')
f.write ('  } // End namespace geometry\n')
f.write ('} // End namespace pcl\n\n')

f.write ('#endif // PCL_GEOMETRY_MESH_INDICES_H\n')

f.close()



# All configuration values have a default; values that are commented out
# serve to show the default.

# -- General configuration -----------------------------------------------------
# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.imgmath', 'sphinxcontrib.doxylink.doxylink']

# Add any paths that contain templates here, relative to this directory.
# templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'Point Cloud Library'
copyright = ''

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '0.0'
# The full version, including alpha/beta/rc tags.
release = '0.0'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
#unused_docs = []

# List of directories, relative to source directory, that shouldn't be searched
# for source files.
exclude_trees = []

# The reST default role (used for this markup: `text`) to use for all documents.
default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = { 'rightsidebar' : 'true' }

# Add any paths that contain custom themes here, relative to this directory.
html_theme_path = ['.']

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
html_short_title = 'Home'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# If false, no module index is generated.
html_use_modindex = False

# If false, no index is generated.
html_use_index = True

# If true, the index is split into individual pages for each letter.
html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
html_file_suffix = '.html'
html_show_copyright = True
html_show_sphinx = True
html_add_permalinks = None
needs_sphinx = '1.0'
file_insertion_enabled = True
raw_enabled = True
# Set up doxylink
doxylink = {'pcl' : ('../../../build/doc/doxygen/pcl.tag', 'http://docs.pointclouds.org/trunk/')}



#/usr/bin/env python

import sys
import Image

# Settings:
# Color of background pixels
backgroundpixel = (255, 255, 255)
# scale the point cloud to fit inside (-1,-1) - (1,1)
scaling = True 

im = Image.open (sys.argv[1])

# resulting point cloud vector
points = []

# minimal/maximal pixel positions that contain non-background
minimum = [d for d in im.size]
maximum = [0, 0]

for x in range (im.size[0]):
  for y in range (im.size[1]):
    px = im.getpixel((x,y))
    if (px != backgroundpixel):
      points.append ([x, y, 0, px[0], px[1], px[2]])
      if maximum[0] < x:
        maximum[0] = x
      if minimum[0] > x:
        minimum[0] = x
      if maximum[1] < y:
        maximum[1] = y
      if minimum[1] > y:
        minimum[1] = y

size_x = maximum[0] - minimum[0]
size_y = maximum[1] - minimum[1]

if (size_x > size_y):
  scale = [1.0, float (size_y) / size_x]
else:
  scale = [float (size_x) / size_y, 1.0]

if scaling:
  #aspect_ratio = 
  for p in points:
    for d in range(2):
      p[d] = scale[d] * ((float (p[d]) - minimum[d]) / (maximum[d] - minimum[d]) * 2.0 - 1.0)

print "VERSION .7"
print "FIELDS x y z rgb "
print "SIZE 4 4 4 4"
print "TYPE F F F F"
print "COUNT 1 1 1 1"
print "WIDTH %i" % len (points)
print "HEIGHT 1"
print "VIEWPOINT 0 0 0 1 0 0 0"
print "POINTS %i" % len (points)
print "DATA ascii"

for p in points:
  print p[0], p[1], p[2], p[3]<<16 | p[4]<<8 | p[5]
